﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 自キャラクタのショット群を扱うクラスです。
    /// </summary>
    class CShotGroup
    {

        /// <summary>
        /// ショット 0の弾数。
        /// </summary>
        private int[] cshot0num = new int[] { 2, 4 };

        /// <summary>
        /// ショット 0の初期 X座標。
        /// </summary>
        private int[] cshot0pos_x = new int[] { -10, 10, -30, 30 };

        /// <summary>
        /// ショット 1の初期 Y座標。
        /// </summary>
        private int[] cshot0pos_y = new int[] { -30, -30, -10, -10 };

        /// <summary>
        /// オプションの発射角度。
        /// </summary>
        private double[] option0ang = new double[] { -Math.PI / 2, -Math.PI / 2, -Math.PI / 2 - Math.PI / 4, -Math.PI / 2 + Math.PI / 4 };

        /// <summary>
        /// 自キャラクタのショット群。
        /// </summary>
        private CShot[] cshots = new CShot[Setting.CSHOT_MAX];

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public CShotGroup()
        {
            for (int i = 0; i < this.cshots.Length; i++)
            {
                this.cshots[i] = new CShot();
            }
        }

        /// <summary>
        /// ショットにアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>ショット</returns>
        public CShot this[int index]
        {
            get
            {
                return this.cshots[index];
            }
        }

        /// <summary>
        /// 自キャラクタのショットの数を返します。
        /// </summary>
        /// <returns>ショットの数</returns>
        public int length()
        {
            return this.cshots.Length;
        }
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.cshots.Length; i++)
                this.cshots[i].initialize();
        }
        
        /// <summary>
        /// 自キャラクタの空いているショットを探します。
        /// </summary>
        /// <returns>空いているショット。無い場合は null</returns>
        private CShot search_cshot()
        {
            for (int i = 0; i < this.cshots.Length; i++)
            {
                CShot cshot = this.cshots[i];
                if (cshot.flag == 0)
                    return cshot;
            }

            return null;
        }

        /// <summary>
        /// 通常ショットを登録します。
        /// </summary>
        private void ch0_shot_pattern()
        {
            int n = this.cshot0num[(GV.ch.power < 200) ? 0 : 1];
            for(int i = 0; i < n; i ++)
            {
                CShot cshot = this.search_cshot();
                if(cshot != null)
                {
                    cshot.flag = 1;
                    cshot.cnt = 0;
                    cshot.angle = -Math.PI / 2;
                    cshot.spd = 20.0;
                    cshot.x = GV.ch.x + this.cshot0pos_x[i];
                    cshot.y = GV.ch.y + this.cshot0pos_y[i];
                    cshot.power = ((n < 2) ? 20 : 12) + GV.ch.power / 100; // 2筋なら 20、4筋なら 12
                    cshot.knd = 0;

                    ImageManager imageManager = ImageManager.GetInstance();
                    cshot.img = imageManager.img_cshot[0];
                }
            }

            MusicManager musicManager = MusicManager.GetInstance();
            musicManager.se_flag[2] = 1; // 発射音をオン

            // オプションショットを登録する
            if (GV.ch.power >= 100)
            {
                for (int i = 0; i < ((GV.ch.power < 300) ? 2 : 4); i++)
                {
                    CShot cshot = this.search_cshot();
                    if (cshot != null)
                    {
                        cshot.flag = 1;
                        cshot.cnt = 0;
                        cshot.angle = option0ang[i];
                        cshot.spd = 20.0;
                        cshot.x = GV.option_bbs[i % 2].x;
                        cshot.y = GV.option_bbs[i % 2].y;
                        cshot.power = 10 - 7 * (i / 2); // マイナスにならないよう注意
                        cshot.knd = 1;

                        ImageManager imageManager = ImageManager.GetInstance();
                        cshot.img = imageManager.img_cshot[1];
                    }
                }
            }
        }

        /// <summary>
        /// 低速通常ショットを登録します。
        /// </summary>
        private void ch1_shot_pattern()
        {
            int n = this.cshot0num[(GV.ch.power < 200) ? 0 : 1];
            for(int i = 0; i < n; i ++)
            {
                CShot cshot = this.search_cshot();
                if(cshot != null)
                {
                    cshot.flag = 1;
                    cshot.cnt = 0;
                    cshot.angle = -Math.PI / 2;
                    cshot.spd = 20.0;
                    cshot.x = GV.ch.x + this.cshot0pos_x[i] / 3; // 低速中なら位置を中心側へ
                    cshot.y = GV.ch.y + this.cshot0pos_y[i] / 2;
                    cshot.power = ((n < 2) ? 20 : 12) + GV.ch.power / 100; // 2筋なら 20、4筋なら 12
                    cshot.knd = 0;

                    ImageManager imageManager = ImageManager.GetInstance();
                    cshot.img = imageManager.img_cshot[0];
                }
            }

            MusicManager musicManager = MusicManager.GetInstance();
            musicManager.se_flag[2] = 1; // 発射音をオン

            // オプションショットを登録する
            if (GV.ch.power >= 100)
            {
                for (int i = 0; i < ((GV.ch.power < 300) ? 2 : 4); i++)
                {
                    CShot cshot = this.search_cshot();
                    if (cshot != null)
                    {
                        cshot.flag = 1;
                        cshot.cnt = 0;
                        cshot.angle = option0ang[i];
                        cshot.spd = 20.0;
                        cshot.x = GV.option_bbs[i % 2].x;
                        cshot.y = GV.option_bbs[i % 2].y;
                        cshot.power = 10 - 7 * (i / 2); // マイナスにならないよう注意
                        cshot.knd = 1;

                        ImageManager imageManager = ImageManager.GetInstance();
                        cshot.img = imageManager.img_cshot[1];
                    }
                }
            }
        }

        /// <summary>
        /// ショットを登録します。
        /// </summary>
        private void enter_shot()
        {
            KeyManager keyManager = KeyManager.GetInstance();
            if (keyManager.CheckStatePad(GV.configpad.shot) > 0) // ショットボタンが押されていたら
            {
                GV.ch.increment_shot_cnt();
                if (GV.ch.shot_cnt % 3 == 0) // 3カウントに 1回
                {
                    if (keyManager.CheckStatePad(GV.configpad.slow) > 0) // 低速移動中なら
                        this.ch1_shot_pattern();
                    else
                        this.ch0_shot_pattern();
                }
            }
            else
                GV.ch.reset_shot_cnt();
        }

        /// <summary>
        /// ショットの移動計算をします。
        /// </summary>
        private void calc_cshot()
        {
            for (int i = 0; i < this.cshots.Length; i++)
            {
                CShot cshot = this.cshots[i];
                if (cshot.flag == 1)
                    cshot.calc_cshot();
            }
        }

        /// <summary>
        /// 自キャラクタのショットのメイン処理。
        /// </summary>
        public void cshot_main()
        {
            this.calc_cshot(); // ショットの軌道計算
            this.enter_shot(); // ショットの登録
        }

        /// <summary>
        /// 自キャラクタのショット群が敵に当たる処理をします。
        /// </summary>
        public void cshot_and_enemy()
        {
            for (int i = 0; i < this.cshots.Length; i++) // 全ショットについて処理する
            {
                CShot cshot = this.cshots[i];
                if(cshot.flag > 0)
                    cshot.cshot_and_enemy(); // 各ショットについて処理する
            }
        }
    }
}
