﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// FPSを管理するクラスです。
    /// </summary>
    class Fps
    {
        /// <summary>
        /// フレーム数。
        /// </summary>
        private const int FLAME = 60;

        /// <summary>
        /// 1周した時間を記録する。
        /// </summary>
        private static int[] f = new int[FLAME];

        /// <summary>
        /// FPSのカウンタ。
        /// </summary>
        private static int fps_count;

        /// <summary>
        /// 60フレームに 1回基準となる時刻を記録する変数。
        /// </summary>
        private static int count0t;

        /// <summary>
        /// 平均 FPS。
        /// </summary>
        private static double ave;

        /// <summary>
        /// 待ち時間。
        /// </summary>
        private static int t;

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private Fps()
        {
            ; // 何もしない
        }

        /// <summary>
        /// 待ちます。
        /// </summary>
        public static void fps_wait()
        {
            int term;
            int gnt;

            if (fps_count == 0) // 60フレームの 1回目なら
            {
                if (t == 0) // 完全に最初なら待たない
                    term=0;
                 else // 前回記録した時間を元に計算
                    term = count0t + 1000 - DX.GetNowCount();
            }
            else // 待つべき時間 = 現在あるべき時刻 - 現在の時刻
                term = (int)(count0t + fps_count * (1000.0 / FLAME)) - DX.GetNowCount();

            if (term>0) // 待つべき時間だけ待つ
                System.Threading.Thread.Sleep(term);

            gnt = DX.GetNowCount();
            if (fps_count == 0) // 60フレームに 1度基準を作る
                count0t = gnt;
            f[fps_count] = gnt - t; //１周した時間を記録
            t = gnt;

            // 平均計算
            if (fps_count == FLAME - 1)
            {
                ave = 0.0;
                for (int i = 0; i < FLAME;i++)
                    ave += f[i];
                ave /= FLAME;
            }

            fps_count++;
            fps_count = fps_count % FLAME;
        }

        /// <summary>
        /// FPSを描画します。
        /// </summary>
        /// <param name="x">X座標</param>
        /// <param name="y">Y座標</param>
        public static void draw_fps(int x, int y)
        {
            if (ave != 0.0)
            {
                double fps = 1000.0 / ave;
                DX.DrawString(x, y, "[" + fps.ToString("f1") + "]", GV.color[0], GV.color[0]);
            }
        }
    }
}
