﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// ボスの出現情報群を管理するクラスです。
    /// </summary>
    class BossInfoGroup
    {
        /// <summary>
        /// ボス情報群。
        /// </summary>
        private BossInfo[] boss_infos = new BossInfo[Setting.BOSS_NUM];

        /// <summary>
        /// 現在のステージに登場するボスの数。
        /// </summary>
        public int max_num;

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public BossInfoGroup()
        {
            for (int i = 0; i < this.boss_infos.Length; i++)
                this.boss_infos[i] = new BossInfo();
        }

        /// <summary>
        /// ボス情報にアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>ボス情報</returns>
        public BossInfo this[int index]
        {
            get
            {
                return this.boss_infos[index];
            }
        }

        /// <summary>
        /// ボス情報の数を返します。
        /// </summary>
        /// <returns>ボス情報の数</returns>
        public int length()
        {
            return this.boss_infos.Length;
        }

        /// <summary>
        /// 読み込みます。
        /// </summary>
        public void load()
        {
            this.max_num = 2;

            // ここは後からきちんとセットします
            // ボスの HP
            int[,] boss_hp = new int[,] {
                { 5000, 10000, -1 },
                { 8000, 20000, 30000 },
            };

            // ボスの背景情報
            int[,] back_knd = new int[,] {
                { 0, 1, -1 },
                { 0, 1, 1 },
            };

            // 弾幕の種類
            int[,] shot_knd = new int[,] {
                { 0, 1, -1 },
                { 2, 3, 4 },
            };

            // 後々きちんと設定しましょう
            this.boss_infos[0].appear_count = 1650; // 中ボスが出現する時刻
            this.boss_infos[1].appear_count = 2850; // 最後ボスが出現する時刻

            this.boss_infos[0].danmaku_num = 2; // 中ボスが何個目までの弾幕を出すか
            this.boss_infos[1].danmaku_num = 3; // 最後に出るボスが何個目までの弾幕を出すか

            for (int j = 0; j < this.max_num; j++)
                for (int i = 0; i < this.boss_infos[j].danmaku_num; i++)
                    this.boss_infos[j].shot_knd[i] = shot_knd[j, i];

            for (int j = 0; j < this.max_num; j++)
                for (int i = 0; i < this.boss_infos[j].danmaku_num; i++)
                this.boss_infos[j].set_hp[i] = boss_hp[j, i];

            for (int j = 0; j < this.max_num; j++)
                for (int i = 0; i < this.boss_infos[j].danmaku_num; i++) // 弾幕それぞれの背景の種類
                    this.boss_infos[j].back_knd[i] = back_knd[j, i];
        }
    }
}
