﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// ボスの弾幕の処理パターンを表すクラスです。
    /// </summary>
    class BossShotPattern
    {
        /// <summary>
        /// 自キャラクタとボスとの成す角度を返します。
        /// </summary>
        /// <returns>角度</returns>
        private static double bossatan2()
        {
            return Math.Atan2(GV.ch.y - GV.boss.y, GV.ch.x - GV.boss.x);
        }

        /// <summary>
        /// 指定座標と弾との成す角度を返します。
        /// </summary>
        /// <param name="bullet">弾</param>
        /// <param name="x">X座標</param>
        /// <param name="y">Y座標</param>
        /// <returns></returns>
        private static double bossatan3(Bullet bullet, double x, double y)
        {
            return Math.Atan2(y - bullet.y, x - bullet.x);
        }

        /// <summary>
        /// 弾幕 000。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH000(BossShot boss_shot)
        {
            const int TM000 = 120;

            int t = boss_shot.cnt % TM000;
            if (t < 60 && t % 10 == 0)
            {
                double angle = bossatan2();
                for (int i = 0; i < 30; i++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = 0;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 8;
                        bullet.angle = angle + MyMath.PI2 / 30 * i;
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.0;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 001用フィールド。ターン番号を記憶。
        /// </summary>
        private static int cnum001;

        /// <summary>
        /// 弾幕 001。サイレントセレナ。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH001(BossShot boss_shot)
        {
            const int TM001 = 60;

            int t = boss_shot.cnt % TM001;
            int t2 = boss_shot.cnt;

            if (t2 == 0) // 最初の初期化
                cnum001 = 0;
            if (t == 0) // 1ターン目の最初の初期化
            {
                boss_shot.base_angle[0] = bossatan2(); // 自キャラクタとボスの角度
                if (cnum001 % 4 == 3) // 4ターンに 1回移動
                    GV.boss.move_boss_pos(40, 30, Setting.FMX - 40, 120, 60.0, 60);
            }

            if (t == TM001 / 2 - 1) // 半分からは
                boss_shot.base_angle[0] += MyMath.PI2 / 20 / 2; // 自キャラクタ狙いを外す

            // 円形弾群の登録
            if (t % (TM001 / 10) == 0) // 1ターンに 10回
            {
                double angle = bossatan2(); // 自キャラクタとボスの角度
                for (int i = 0; i < 20; i++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = 1; // 原作では 4に設定されているが、プレイ動画の色から判断すると 1。
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 8;
                        bullet.angle = boss_shot.base_angle[0] + MyMath.PI2 / 20 * i; // ベース角度から 20個回転して発射
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 2.7; // スピード

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }

            // 4カウントに 1回、下に落ちる弾を登録
            if (t % 4 == 0)
            {
                Bullet bullet = boss_shot.search_boss_shot();
                if (bullet != null)
                {
                    bullet.col = 0;
                    bullet.x = MyMath.GetRand(Setting.FMX);
                    bullet.y = MyMath.GetRand(200);
                    bullet.knd = 8;
                    bullet.angle = Math.PI / 2; // 真下の角度
                    bullet.flag = 1;
                    bullet.cnt = 0;
                    bullet.spd = 1 + MyMath.rang(0.5);

                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            if (t == TM001 - 1)
                cnum001++;
        }

        /// <summary>
        /// 弾幕 002。パーフェクトシリーズ。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH002(BossShot boss_shot)
        {
            const int TM002 = 650;

            int t = boss_shot.cnt % TM002;
            if (t == 0 || t == 210)
            {
                // 40 < x < FMX - 40、50 < y < 150 の範囲で 100離れた位置に 80カウントで移動する
                GV.boss.move_boss_pos(40, 50, Setting.FMX, 150, 100, 80);
            }

            // 最初のランダム発射
            if (t < 180)
            {
                for (int i = 0; i < 2; i++) // 1カウントに 2回発射
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = MyMath.GetRand(6); // 7種類の色をランダムに
                        bullet.x = GV.boss.x; // 発射初期座標はボスの位置
                        bullet.y = GV.boss.y;
                        bullet.knd = 7; // 弾の種類
                        bullet.angle = MyMath.rang(MyMath.PI2 / 20) + MyMath.PI2 / 10 * t;
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.2 + MyMath.rang(2.1);
                        bullet.state = 0; // 弾のステータス
                        bullet.kaiten = 1; // 弾を回転するかどうかのフラグ
                    }
                }
                if (t % 10 == 0)
                {
                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            // 自キャラクタの位置に依存する 8方向発射
            if (210 < t && t < 270 && t % 3 == 0)
            {
                double angle = bossatan2();
                for (int i = 0; i < 8; i++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = 0;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 7;
                        // 自キャラクタとボスとの成す角を基準に 8方向に発射する
                        bullet.angle = angle - Math.PI / 2 * 0.8 + Math.PI * 0.8 / 7 * i + MyMath.rang(Math.PI / 180);
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.0 + MyMath.rang(0.3);
                        bullet.state = 2; // 弾のステータス
                        bullet.kaiten = 1;
                    }
                }
                if (t % 10 == 0)
                {
                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            for (int i = 0; i < boss_shot.bullets.length(); i++)
            {
                Bullet bullet = boss_shot.bullets[i];
                if (bullet.flag > 0)
                {
                    // tが 190の時にすべてストップさせ、白くし、カウントリセット
                    if (bullet.state == 0)
                    {
                        if (t == 190)
                        {
                            bullet.kaiten = 0; // 回転を止める
                            bullet.spd = 0.0;
                            bullet.col = 9; // 本来は 9を設定したいのだけど暫定的に 8を設定
                            bullet.cnt = 0; // カウントリセット
                            bullet.state = 1; // ステータスを 1に
                        }
                    }
                    if (bullet.state == 1)
                    {
                        if (bullet.cnt == 200)
                        {
                            bullet.angle = MyMath.rang(Math.PI); // 全方向ランダム
                            bullet.kaiten = 1; // 回転するフラグを立てる
                        }
                        if (bullet.cnt > 200)
                            bullet.spd += 0.01; // 段々加速
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 003用フィールド。tcnt。
        /// </summary>
        private static int tcnt003;

        /// <summary>
        /// 弾幕 003用フィールド。cnt。
        /// </summary>
        private static int cnt003;

        /// <summary>
        /// 弾幕 003用フィールド。cnum;
        /// </summary>
        private static int cnum003;

        /// <summary>
        /// 弾幕 003。恋の迷路。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH003(BossShot boss_shot)
        {
            const int TM003 = 600;
            const int DF003 = 20;

            int t = boss_shot.cnt % TM003;
            int t2 = boss_shot.cnt;

            bool cnum_flag = cnum003 % 2 != 0; // 原作版と異なり 2の剰余を取っています

            if (t2 == 0)
            {
                GV.boss.phy.input_phy_pos(GV.boss, Setting.FMX / 2, Setting.FMY / 2, 50);
                cnum003 = 0;
            }
            if (t == 0)
            {
                boss_shot.base_angle[0] = bossatan2();
                cnt003 = 0;
                tcnt003 = 2;
            }
            if (t < 540 && t % 3 == 0)
            {
                double angle = bossatan2();
                if (tcnt003 - 2 == cnt003 || tcnt003 - 1 == cnt003)
                {
                    if (tcnt003 - 1 == cnt003)
                    {
                        boss_shot.base_angle[1] = boss_shot.base_angle[0] + MyMath.PI2 / DF003 * cnt003 * (cnum_flag ? -1 : 1)
                            - MyMath.PI2 / (DF003 * 6) * 3;
                        tcnt003 += DF003 - 2;
                    }
                }
                else
                {
                    for (int i = 0; i < 6; i++)
                    {
                        Bullet bullet = boss_shot.search_boss_shot();
                        if (bullet != null)
                        {
                            bullet.col = cnum_flag ? 5 : 1; // 原作版と色を変えています
                            bullet.x = GV.boss.x;
                            bullet.y = GV.boss.y;
                            bullet.knd = 8;
                            bullet.angle = boss_shot.base_angle[0]
                                + MyMath.PI2 / DF003 * cnt003 * (cnum_flag ? -1 : 1)
                                + MyMath.PI2 / (DF003 * 6) * i * (cnum_flag ? -1 : 1);
                            bullet.flag = 1;
                            bullet.cnt = 0;
                            bullet.spd = 2.0;

                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;
                        }
                    }
                }
                cnt003++;
            }
            if (40 < t && t < 540 && t % 30 == 0)
            {
                double angle;
                for (int j = 0; j < 3; j++)
                {
                    angle = boss_shot.base_angle[1] - MyMath.PI2 / 36 * 4;
                    for (int i = 0; i < 27; i++)
                    {
                        Bullet bullet = boss_shot.search_boss_shot();
                        if (bullet != null)
                        {
                            bullet.col = cnum_flag ? 6 : 0;
                            bullet.x = GV.boss.x;
                            bullet.y = GV.boss.y;
                            bullet.knd = 7;
                            bullet.angle = angle;
                            bullet.flag = 1;
                            bullet.cnt = 0;
                            bullet.spd = 4.0 - 1.6 / 3 * j;

                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;
                        }
                        angle -= MyMath.PI2 / 36;
                    }
                }
            }
            if (t == TM003 - 1)
                cnum003++;
        }

        /// <summary>
        /// 弾幕 004用フィールド。tm。
        /// </summary>
        private static int tm004;

        /// <summary>
        /// 弾幕 004。ケロちゃん風雨に負けず。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH004(BossShot boss_shot)
        {
            const int TM004 = 200;

            int t = boss_shot.cnt % TM004;
            int t2 = boss_shot.cnt;

            if (t == 0) // 周期の最初に tmをセット
                tm004 = 190 + (int)MyMath.rang(30);

            double angle = Math.PI * 1.5 + Math.PI / 6 * Math.Sin(MyMath.PI2 / tm004 * t2);

            // 4カウントに 1回上に 8way弾を撃つ
            if (t2 % 4 == 0)
            {
                for (int n = 0; n < 8; n++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.flag = 1;
                        bullet.state = 0;
                        bullet.cnt = 0;
                        bullet.knd = 4;
                        bullet.col = 0;
                        bullet.angle = 0.0;
                        bullet.eff_detail = 1;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.vx = Math.Cos(angle - Math.PI / 8 * 4 + Math.PI / 8 * n + Math.PI / 16) * 3;
                        bullet.vy = Math.Sin(angle - Math.PI / 8 * 4 + Math.PI / 8 * n + Math.PI / 16) * 3;
                    }
                }
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[0] = 1;
            }
            if (t % 1 == 0 && t2 > 80)
            {
                int num = 1;
                if (t % 2 == 0)
                    num = 2;
                for (int n = 0; n < num; n++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        angle = Math.PI * 1.5 - Math.PI / 2 + Math.PI / 12 * (t2 % 13) + MyMath.rang(Math.PI / 15);
                        bullet.flag = 1;
                        bullet.state = 1;
                        bullet.cnt = 0;
                        bullet.knd = 8;
                        bullet.eff_detail = 0;
                        bullet.col = 1; // 原作版ソースでは 4のところを 1
                        bullet.angle = 0.0;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.vx = Math.Cos(angle) * 1.4 * 1.2;
                        bullet.vy = Math.Sin(angle) * 1.4;
                    }
                }
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[7] = 1;
            }
            for (int i = 0; i < boss_shot.bullets.length(); i++)
            {
                Bullet bullet = boss_shot.bullets[i];
                if (bullet != null)
                {
                    if (bullet.state == 0)
                    {
                        if (bullet.cnt < 150)
                            bullet.vy += 0.03;
                        bullet.x += bullet.vx;
                        bullet.y += bullet.vy;
                    }
                    if (bullet.state == 1)
                    {
                        if (bullet.cnt < 160)
                            bullet.vy += 0.03;
                        bullet.x += bullet.vx;
                        bullet.y += bullet.vy;
                        bullet.angle = Math.Atan2(bullet.vy, bullet.vx);
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 005用フィールド。sst。
        /// </summary>
        private static int sst005;

        /// <summary>
        /// 弾幕 005用フィールド。bnum。
        /// </summary>
        private static int bnum005;

        /// <summary>
        /// 弾幕 005用フィールド。sx。
        /// </summary>
        private static double sx005;

        /// <summary>
        /// 弾幕 005用フィールド。sy。
        /// </summary>
        private static double sy005;

        /// <summary>
        /// 弾幕 005用フィールド。sangle。
        /// </summary>
        private static double sangle005;
        
        /// <summary>
        /// 弾幕 005。早苗風☆弾。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH005(BossShot boss_shot)
        {
            const int TM005 = 820;
            const double RANGE005 = 100.0;
            const int LEM005 = 4;

            int t = boss_shot.cnt % TM005;
            int t2 = boss_shot.cnt;

            if (t == 0) // 1周期の最初なら
            {
                sst005 = 0;
                sx005 = GV.boss.x;
                sy005 = GV.boss.y - RANGE005;
                sangle005 = Math.PI / 5 / 2 + Math.PI / 2;
            }

            // 星を描く
            if (sst005 <= 4)
            {
                for (int i = 0; i < 2; i++)
                {
                    sx005 += Math.Cos(sangle005) * LEM005; // 座標を計算
                    sy005 += Math.Sin(sangle005) * LEM005;

                    // 円と交わったら
                    if ((sx005 - GV.boss.x) * (sx005 - GV.boss.x) + (sy005 - GV.boss.y) * (sy005 - GV.boss.y) > RANGE005 * RANGE005)
                    {
                        sangle005 -= Math.PI - Math.PI / 5; // 方向転換
                        sst005++; // ステータス変換
                        if (sst005 == 5) // ステータスが 5なら
                            break; // 終わる
                    }
                    for (int j = 0; j < 5; j++) // 星を 5つ描く
                    {
                        Bullet bullet = boss_shot.search_boss_shot();
                        if (bullet != null)
                        {
                            bullet.flag = 1;
                            bullet.state = j;
                            bullet.cnt = 0;
                            bullet.spd = 0.0;
                            bullet.knd = 10;
                            bullet.col = j;
                            bullet.angle = -Math.PI / 2 + MyMath.PI2 / 5 * j;
                            bullet.x = sx005;
                            bullet.y = sy005;
                            bullet.vx = Math.Cos(sangle005) * 1.4 * 1.2;
                            bullet.vy = Math.Sin(sangle005) * 1.4;
                            bullet.base_angle[0] = sangle005 - Math.PI + Math.PI / 20 * bnum005;
                        }
                    }
                    bnum005++;
                }
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[0] = 1;
            }

            for (int i = 0; i < boss_shot.bullets.length(); i++)
            {
                Bullet bullet = boss_shot.bullets[i];
                if (bullet.flag > 0)
                {
                    int cnt = bullet.cnt;

                    if (bullet.state < 10) // ステータスが 10未満なら
                    {
                        if (t == 150) // 150なら星を 5方向に発射
                        {
                            bullet.spd = 4.0; // スピード
                            bullet.cnt = 0; // カウンタリセット
                            bullet.state += 10; // ステータス番号変更
                        }
                    }
                    else if (bullet.state < 20) // ステータスが 10～19なら
                    {
                        if (cnt <= 80) // 減速
                            bullet.spd -= 0.05;
                        if (cnt == 100) // セットした基準角度から綺麗な曲線を描く
                            bullet.angle = bullet.base_angle[0];
                        if (cnt >= 100 && cnt < 160) // 加速
                            bullet.spd += 0.015;
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 006用フィールド。num。
        /// </summary>
        private static int num006;

        /// <summary>
        /// 弾幕 006。そして誰もいなくなるか? 風弾幕。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH006(BossShot boss_shot)
        {
            const int TM006 = 300;
            const int DNUMX = 15; // 横一列に置く弾の数

            int t = boss_shot.cnt % TM006;
            int t2 = boss_shot.cnt;
            double xlen = (double)Setting.FMX / DNUMX; // 画面に DNUMX個並べるには xlen感覚で並べれば良い
            int dnumy = Setting.FMY / (int)xlen; // xlenずつ並べると鉛直方向にいくつはいるか

            if (dnumy == 0)
            {
                Debug.WriteLine("boss_shot_bulletH006で 0割り");
                return;
            }

            double ylen = (double)Setting.FMY / dnumy; // ylenずつ置けば良い

            if (t2 == 0)
                num006 = 0;
            if (t == 0)
            {
                double x = 0.0;
                double y = 0.0;
                double angle = 0.0;
                int cnum = 0;

                for (int j = 0; j < 4; j++) // 画面左上端から、右→下→左→上の 4方向移動させる
                {
                    int to = (j % 2 != 0) ? dnumy : DNUMX;
                    for (int i = 0; i <= to; i++) // 水平方向、垂直方向では置く個数が違うので to回置く
                    {
                        Bullet bullet = boss_shot.search_boss_shot();
                        if (bullet != null)
                        {
                            bullet.x = x; // 座標
                            bullet.y = y;
                            bullet.knd = 4; // 種類
                            bullet.flag = 1;
                            bullet.cnt = 0;
                            switch (num006)
                            {
                                case 0: // 発射種類 1
                                    bullet.col = 0; // 弾の色
                                    bullet.angle = bossatan3(bullet, Setting.FMX / 2, Setting.FMY / 2); // 角度
                                    bullet.spd = 1.3; // スピード
                                    bullet.state = 0; // ステータス
                                    break;
                                case 1: // 発射種類 2
                                    bullet.col = 3; // 弾の色
                                    bullet.angle = bossatan3(bullet, Setting.FMX / 2, Setting.FMY / 2); // 角度
                                    bullet.spd = 1.4
                                        + (((j % 2 != 0) ? -1 : 1) * ((Math.Cos(MyMath.PI2 / to * i - Math.PI) + 1) / 2)) * 0.4; // スピード
                                    bullet.state = 1; // ステータス
                                    break;
                                case 2: // 発射種類 3
                                    bullet.col = 6; // 弾の色
                                    bullet.angle = bossatan3(bullet, Setting.FMX / 2, Setting.FMY / 2); // 角度
                                    bullet.spd = 1.3; // スピード
                                    bullet.state = 2; // ステータス
                                    bullet.base_angle[0] = Math.PI / 1000 * ((j % 2 != 0) ? -1 : 1) * ((Math.Cos(MyMath.PI2 / to * i - Math.PI) + 1) / 2); // 加算する角度
                                    break;
                            }
                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;

                            cnum++;
                        }
                        x += Math.Cos(angle) * xlen;
                        y += Math.Sin(angle) * xlen;
                    }
                    angle += Math.PI / 2;
                }
            }

            for (int i = 0; i < boss_shot.bullets.length(); i++)
            {
                Bullet bullet = boss_shot.bullets[i];
                if (bullet.flag > 0)
                {
                    int cnt = bullet.cnt;
                    switch (bullet.state)
                    {
                        case 2: // 発射種類 3なら
                            bullet.angle += bullet.base_angle[0]; // 角度を計算
                            break;
                        default:
                            break;
                    }
                }
            }

            if (t == TM006 - 1)
            {
                num006++;
                num006 = num006 % 3;
            }
        }

        /// <summary>
        /// 弾幕 007用フィールド。num。
        /// </summary>
        private static int num007;

        /// <summary>
        /// 弾幕 007。そして誰もいなくなるか? 風弾幕。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH007(BossShot boss_shot)
        {
            const int TM007 = 300;
            const int DNUMX = 9; // 横一列に置く弾の数

            int t = boss_shot.cnt % TM007;
            int t2 = boss_shot.cnt;
            double xlen = (double)Setting.FMX / DNUMX; // 画面に DNUMX個並べるには xlen感覚で並べれば良い
            int dnumy = Setting.FMY / (int)xlen; // xlenずつ並べると鉛直方向にいくつはいるか

            if (dnumy == 0)
            {
                Debug.WriteLine("boss_shot_bulletH007で 0割り");
                return;
            }

            double ylen = (double)Setting.FMY / dnumy; // ylenずつ置けば良い

            if (t2 == 0)
                num007 = 0;
            if (t == 0)
            {
                double x = 0.0;
                double y = 0.0;
                double angle = 0.0;
                int cnum = 0;

                for (int j = 0; j < 4; j++) // 画面左上端から、右→下→左→上の 4方向移動させる
                {
                    int to = (j % 2 != 0) ? dnumy : DNUMX;
                    for (int i = 0; i <= to; i++) // 水平方向、垂直方向では置く個数が違うので to回置く
                    {
                        for (int s = 0; s < 2; s++)
                        {
                            Bullet bullet = boss_shot.search_boss_shot();
                            if (bullet != null)
                            {
                                bullet.x = x; // 座標
                                bullet.y = y;
                                bullet.knd = 4; // 種類
                                bullet.flag = 1;
                                bullet.cnt = 0;
                                switch (num007)
                                {
                                    case 0: // 直線
                                        bullet.col = 0; // 弾の色
                                        bullet.angle = angle + Math.PI / 2; // 角度
                                        bullet.spd = 1.1 + 0.5 * s; // スピード
                                        bullet.state = 0; // ステータス
                                        break;
                                    case 1: // 斜め交差
                                        bullet.col = 3; // 弾の色
                                        bullet.angle = angle + Math.PI / 2 - Math.PI / 14 + Math.PI / 7 * s; // 角度
                                        bullet.spd = 1.3;
                                        bullet.state = 1; // ステータス
                                        break;
                                }
                                MusicManager musicManager = MusicManager.GetInstance();
                                musicManager.se_flag[0] = 1;

                                cnum++;
                            }
                        }
                        x += Math.Cos(angle) * xlen;
                        y += Math.Sin(angle) * xlen;
                    }
                    angle += Math.PI / 2;
                }
            }

            if (t == TM007 - 1)
            {
                num007++;
                num007 = num007 % 2;
            }
        }

        /// <summary>
        /// 弾幕 008用フィールド。num。
        /// </summary>
        private static int num008;

        /// <summary>
        /// 弾幕 008。レーザーのサンプル。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH008(BossShot boss_shot)
        {
            const int TM008 = 420;
            const int DIST = 60;

            int t = boss_shot.cnt % TM008;
            int t2 = boss_shot.cnt;

            if (t2 == 0)
                num008 = 4;
            if (t == 0)
            {
                for (int j = 0; j < 2; j++)
                {
                    for (int i = 0; i < num008; i++)
                    {
                        int plmn = (j != 0) ? -1 : 1;
                        Lazer lazer = GV.lazers.search_lazer();
                        if (lazer != null)
                        {
                            lazer.col = j; // レーザーの色
                            lazer.knd = 0; // レーザーの種類
                            lazer.angle = MyMath.PI2 / num008 * i + MyMath.PI2 / (num008 * 2) * MyMath.PI2 / (num008 * 4) * ((num008 + 1) % 2); // 角度
                            lazer.startpt.x = GV.boss.x + Math.Cos(lazer.angle) * DIST; // 座標
                            lazer.startpt.y = GV.boss.y + Math.Sin(lazer.angle) * DIST;
                            lazer.flag = 1;
                            lazer.cnt = 0;
                            lazer.haba = 2.0; // 幅
                            lazer.state = j; // ステータス
                            lazer.length = 240.0; // レーザーの長さ
                            lazer.hantei = 0;
                            lazer.lphy.conv_flag = 1; // 回転フラグ
                            lazer.lphy.conv_base_x = GV.boss.x; // 回転基準位置
                            lazer.lphy.conv_base_y = GV.boss.y;
                            lazer.lphy.conv_x = lazer.startpt.x; // 回転元の位置
                            lazer.lphy.conv_y = lazer.startpt.y;
                            lazer.input_lphy(80, Math.PI / num008 * plmn); // 代入
                        }
                    }
                }
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[33] = 1;
            }

            // レーザー計算
            for (int i = 0; i < GV.lazers.length(); i++)
            {
                Lazer lazer = GV.lazers[i];
                if (lazer.flag > 0)
                {
                    int cnt = lazer.cnt;
                    int state = lazer.state;
                    if (state == 0 || state == 1)
                    {
                        if (cnt == 80)
                        {
                            lazer.haba = 30; // 幅を 30に
                            lazer.hantei = 0.5; // 表示幅の半分を判定範囲に
                        }
                        if (cnt >= 260 && cnt <= 320)
                        {
                            if (cnt == 280)
                                lazer.hantei = 0;
                            lazer.haba = 10 * (60 - (cnt - 260)) / 60.0;
                            if (cnt == 320)
                                lazer.flag = 0;
                        }
                    }
                }
            }

            if (t == TM008 - 1)
                num008++;
        }

        /// <summary>
        /// 弾幕 009用フィールド。num。
        /// </summary>
        private static int num009;

        /// <summary>
        /// 弾幕 009。反魂蝶～八部咲き～。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH009(BossShot boss_shot)
        {
            const int TM009 = 420;
            const int DIST = 60;

            int t = boss_shot.cnt % TM009;
            int t2 = boss_shot.cnt;

            if (t2 == 0)
                num009 = 4;
            if (t == 0)
            {
                for (int j = 0; j < 2; j++)
                {
                    for (int i = 0; i < num009; i++)
                    {
                        int plmn = (j != 0) ? -1 : 1;
                        Lazer lazer = GV.lazers.search_lazer();
                        if (lazer != null)
                        {
                            lazer.col = j; // 弾の色
                            lazer.knd = 0; // 弾の種類
                            lazer.angle = MyMath.PI2 / num009 * i + MyMath.PI2 / (num009 * 2) * j + MyMath.PI2 / (num009 * 4) * ((num009 + 1) % 2); // 角度
                            lazer.startpt.x = GV.boss.x + Math.Cos(lazer.angle) * DIST; // 描画し始める座標
                            lazer.startpt.y = GV.boss.y + Math.Sin(lazer.angle) * DIST;
                            lazer.flag = 1; // 表示するか。0:しない、1:する
                            lazer.cnt = 0;
                            lazer.haba = 2; //幅
                            lazer.state = j; // 状態
                            lazer.length = 450; // 長さ(原作ソースコードでは 210)
                            lazer.hantei = 0; // 当たり判定をするか。0:しない、1:する
                            lazer.lphy.conv_flag = 1; // 回転するか。0:しない、1:する
                            lazer.lphy.conv_base_x = GV.boss.x; // 回転の基準となる座標
                            lazer.lphy.conv_base_y = GV.boss.y;
                            lazer.lphy.conv_x = lazer.startpt.x; // 回転するレーザーの位置
                            lazer.lphy.conv_y = lazer.startpt.y;
                            lazer.input_lphy(80, Math.PI / num009 * plmn);//k番目のレーザーを、80カウントで、PI/num*plmnの角度だけ回転する情報を登録
                        }
                    }
                }
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[33] = 1;
            }

            if (t == 50) // 50カウントのとき
            {
                double angle = MyMath.rang(Math.PI); // 自キャラクタとボスとの成す角
                for (int s = 0; s < 2; s++)
                {
                    for (int tt = 0; tt < 3; tt++) // 原作ソースだと変数名がバッティングしている
                    {
                        for (int j = 0; j < 3; j++)
                        {
                            for (int i = 0; i < 30; i++)
                            {
                                Bullet bullet = boss_shot.search_boss_shot();
                                if (bullet != null)
                                {
                                    bullet.col = s; //弾の色
                                    bullet.x = GV.boss.x; //座標
                                    bullet.y = GV.boss.y;
                                    bullet.knd = 11; //弾の種類
                                    bullet.angle = angle + MyMath.PI2 / 30 * i + MyMath.PI2 / 60 * s; //角度
                                    bullet.flag = 1;
                                    bullet.cnt = 0;
                                    bullet.spd = 1.8 - 0.2 * j + 0.1 * s; //スピード
                                    bullet.eff = 0;
                                    bullet.state = tt;
                                }
                                MusicManager musicManager = MusicManager.GetInstance();
                                musicManager.se_flag[0] = 1;
                            }
                        }
                    }
                }
            }

            if (t >= 170 && t < 310 && (t - 170) % 35 == 0)
            {
                int div = ((t - 170) % 70 == 0) ? -1 : 1;
                double angle = MyMath.rang(Math.PI); // 自キャラクタとボスとの成す角
                for (int s = 0; s < 2; s++) // //速度の違う2つの弾がある
                {
                    for (int tt = 0; tt < 3; tt++) //1箇所から3つにわかれる
                    {
                        for (int i = 0; i < 30; i++) // 1周 30個
                        {
                            Bullet bullet = boss_shot.search_boss_shot();
                            if (bullet != null)
                            {
                                bullet.col = 2; // 弾の色
                                bullet.x = GV.boss.x; // 座標
                                bullet.y = GV.boss.y;
                                bullet.knd = 11; // 弾の種類
                                bullet.angle = angle + MyMath.PI2 / 30 * i; // 角度
                                bullet.flag = 1;
                                bullet.cnt = 0;
                                bullet.spd = 2 - 0.3 * s; // スピード
                                bullet.eff = 0;
                                bullet.state = 10 + tt;
                                bullet.base_angle[0] = Math.PI / 300 * div;
                            }
                        }
                    }
                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            if (t == 360)
            {
                double angle = MyMath.rang(Math.PI); // 自キャラクタとボスとの成す角
                for (int tt = 0; tt < 3; tt++) // 1箇所から 3つに分かれる
                {
                    for (int i = 0; i < 30; i++)
                    {
                        Bullet bullet = boss_shot.search_boss_shot();
                        if (bullet != null)
                        {
                            bullet.col = 1; // 弾の色
                            bullet.x = GV.boss.x; // 座標
                            bullet.y = GV.boss.y;
                            bullet.knd = 0; // 弾の種類
                            bullet.angle = angle + MyMath.PI2 / 30 * i; // 角度
                            bullet.flag = 1;
                            bullet.cnt = 0;
                            bullet.spd = 1.8; // スピード
                            bullet.eff = 1;
                            bullet.state = 20 + tt;
                        }
                    }
                }
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[0] = 1;
            }
            
            // 弾の処理
            for (int i = 0; i < boss_shot.bullets.length(); i++)
            {
                Bullet bullet = boss_shot.bullets[i];
                if (bullet.flag > 0)
                {
                    int cnt = bullet.cnt;
                    int state = bullet.state;
                    if (state % 10 == 0)
                    {
                        if (cnt > 90 && cnt <= 100)
                            bullet.spd -= bullet.spd / 220;
                    }
                    if (state % 10 == 1)
                    {
                        if (cnt > 50)
                            bullet.spd += bullet.spd / 45;
                    }
                    if (state % 10 == 2)
                    {
                        if (cnt > 65)
                            bullet.spd += bullet.spd / 90;
                    }
                    if (10 <= state && state <= 12)
                    {
                        if (cnt > 15 && cnt <= 80)
                            bullet.angle += bullet.base_angle[0];
                    }
                }
            }

            // レーザーの処理
            for (int i = 0; i < GV.lazers.length(); i++)
            {
                Lazer lazer = GV.lazers[i];
                if (lazer.flag > 0)
                {
                    int cnt = lazer.cnt;
                    int state = lazer.state;
                    if (state == 0 || state == 1)
                    {
                        if (cnt == 80)
                        {
                            lazer.haba = 10;
                            lazer.hantei = 0.5;
                        }
                        if (cnt >= 260 && cnt <= 320)
                        {
                            if (cnt == 280)
                                lazer.hantei = 0;
                            lazer.haba = 10 * (60 - (cnt - 260)) / 60.0;
                            if (cnt == 320)
                                lazer.flag = 0;
                        }
                    }
                }
            }
            if (t == TM009 - 1)
                num009 ++;
        }

        /// <summary>
        /// 弾幕 010用フィールド。num。
        /// </summary>
        private static int num010;

        /// <summary>
        /// 弾幕 010用フィールド。flag。
        /// </summary>
        private static int flag010;

        /// <summary>
        /// 弾幕 010用フィールド。knum。
        /// </summary>
        private static int knum010;

        /// <summary>
        /// 弾幕 010用フィールド。child_dist。
        /// </summary>
        private static double child_dist010;

        /// <summary>
        /// 弾幕 010用フィールド。child_angle。
        /// </summary>
        private static double child_angle010;

        /// <summary>
        /// 弾幕 010用フィールド。child_dist2。
        /// </summary>
        private static double child_dist2_010;

        /// <summary>
        /// 弾幕 010用フィールド。child_angle2。
        /// </summary>
        private static double child_angle2_010;

        /// <summary>
        /// 弾幕 010。深弾幕結界。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH010(BossShot boss_shot)
        {
            const int TM010 = 9000;
            const double DIST010 = Setting.FMX / 2 * 1.18;
            const double DIST0101 = Setting.FMX / 2 * 0.95;
            const double DIST010_2_0 = Setting.FMX / 2;
            const double DIST010_2_1 = Setting.FMY / 2;
            const int HANSHU = 120; // 子どもが半周する時間
            const int HANSHU1 = 180; // 子どもが半周する時間
            const int HANSHU2_0 = 180; // 子どもが半周する時間
            const int HANSHU2_1 = 120; // 子ども半周する時間
            const int GOOUT010 = 90; // 子どもが中央から外へ行く時間
            const double KAISHI010 = HANSHU;
            const double KAISHI010_1 = HANSHU1 * 0.6;
            const double CHILD_TIME = HANSHU * 5 + HANSHU / 3; // 子どもが存在する時間
            const double CHILD_TIME1 = HANSHU1 * 3 + HANSHU1 * 0.4; // 子どもが存在する時間
            const double CHILD_SHOT_TIME = CHILD_TIME - KAISHI010; // 子どもが実際に発射している時間
            const double CHILD_SHOT_TIME1 = CHILD_TIME1 - KAISHI010_1; // 子どもが実際に発射している時間
            const double ANG0 = Math.PI / 9;
            const double ANG1 = Math.PI / 6;
            const int TIME1 = 900;
            const int RAG = 20;
            const int TERM0 = 20;
            const int ST_ED = 130;
            const double ST0 = CHILD_SHOT_TIME + TERM0;
            const double ED0 = CHILD_SHOT_TIME + TERM0 + ST_ED;
            const int TERM1 = 47;
            const int ST_ED1 = 105;
            const double ST1 = CHILD_SHOT_TIME1 + TERM1;
            const double ED1 = CHILD_SHOT_TIME1 + TERM1 + ST_ED1;

            int t = boss_shot.cnt % TM010;
            int t2 = boss_shot.cnt;
            int tt1 = boss_shot.cnt - TIME1;

            if (t2 == 0)//最初なら
            {
                GV.boss.phy.input_phy_pos(GV.boss, Setting.FMX / 2, Setting.FMY / 2, 50);
                num010 = -1;
                flag010 = 0;
            }

            // 周期の最初
            if (t == 0 || t2 == TIME1)
            {
                num010++;

                // 子どもデータの初期化
                child_dist010 = 0;
                child_angle010 = 0;
                child_dist2_010 = 0;
                child_angle2_010 = 0;
                knum010 = 0;
            }

            // 子どもの登録
            if (t2 == GOOUT010 || t2 == TIME1)
            {
                for (int i = 0; i < 2; i++)
                {
                    Child child = GV.children[i]; // 注意! 子どもを直接、指定している
                    child.flag = 1;
                    child.x = GV.boss.x;
                    child.y = GV.boss.y;
                    child.range = 0.5;
                    child.spd = 1.0;
                    child.angle = 0.0;
                    child.knd = 0;
                    child.col = 0;
                    child.cnt = 0;
                    child.state = i;
                }
            }

            // 1周目
            if (num010 == 0)
            {
                if (GOOUT010 <= t)
                {
                    // 出現してから外向きに広がっていくまで
                    if (GOOUT010 <= t && t < GOOUT010 + KAISHI010)
                        child_dist010 += DIST010 / KAISHI010;
                    //常に回転
                    child_angle010 += Math.PI / HANSHU;
                }
            }

            // 2周目
            if (num010 == 1)
            {
                // 出現してから外向きに広がっていくまで
                if (GV.children[0].cnt < KAISHI010_1)
                    child_dist010 += DIST0101 / KAISHI010_1;
                // 常に回転
                child_angle010 -= Math.PI / HANSHU1;
            }

            // 子どものデータ計算
            for (int i = 0; i < GV.children.length(); i++)
            {
                Child child = GV.children[i];
                if (child.flag > 0) // 登録されていたら
                {
                    // 軌道計算
                    child.x = GV.boss.x + Math.Cos(child_angle010 + Math.PI * child.state) * child_dist010;
                    child.y = GV.boss.y + Math.Sin(child_angle010 + Math.PI * child.state) * child_dist010;

                    // 1回目
                    if (num010 == 0)
                    {
                        if ((int)KAISHI010 < child.cnt) // 発射開始カウント以上なら
                        {
                            if (((t + 6) % 36) / 4 <= 5 && t % 4 == 0) // この時、弾登録
                            {
                                for (int j = 0; j < 3; j++) // 外向き 3way発射
                                {
                                    int k = knum010;
                                    knum010++;

                                    Bullet bullet = GV.boss_shot.bullets[k]; // 注意! 弾を直接、指定している
                                    bullet.col = child.state; // 弾の色
                                    bullet.x = child.x; // 座標
                                    bullet.y = child.y;
                                    bullet.knd = 6;//弾の種類
                                    bullet.angle
                                        = bossatan3(bullet, GV.boss.x, GV.boss.y) + Math.PI - Math.PI / 6 + Math.PI / 6 * j; // 角度
                                    bullet.flag = 1;
                                    bullet.cnt = 0;
                                    bullet.state = 10 + j + 10 * child.state; // 状態
                                    bullet.spd = 0.4; // スピード

                                    MusicManager musicManager = MusicManager.GetInstance();
                                    musicManager.se_flag[0] = 1;
                                }
                            }
                            else if (t % 4 == 0) // それ以外の時で、4回に一回
                            {
                                for (int j = 0; j < 3; j++)
                                {
                                    int k = knum010;
                                    knum010++;

                                    Bullet bullet = GV.boss_shot.bullets[k]; // 注意! 弾を直接、指定している
                                    bullet.col = child.state; // 弾の色
                                    bullet.x = child.x; // 座標
                                    bullet.y = child.y;
                                    bullet.knd = 6; // 弾の種類
                                    bullet.angle
                                        = bossatan3(bullet, GV.boss.x, GV.boss.y) + Math.PI - Math.PI / 6 + Math.PI / 6 * j; // 角度
                                    bullet.flag = 1;
                                    bullet.cnt = 0;
                                    bullet.state = 30;
                                    bullet.spd = 0.4; // スピード
                                }
                            }
                        }
                        //消える時間なら消す
                        if (child.cnt > (int)CHILD_TIME)
                            child.flag = 0;
                    }
                    // 1回目end

                    // 2回目
                    if (num010 == 1)
                    {
                        if ((int)KAISHI010_1 < child.cnt) // 発射カウント以上なら
                        {
                            if ((tt1 - 55) % (3 * 22) < (3 * 15) && t % 3 == 0) // この時登録
                            {
                                for (int j = 0; j < 3; j++) // 外向き発射
                                {
                                    int k = knum010;
                                    knum010++;

                                    Bullet bullet = GV.boss_shot.bullets[k]; // 注意! 弾を直接、指定している
                                    bullet.col = child.state; // 弾の色
                                    bullet.x = child.x; // 座標
                                    bullet.y = child.y;
                                    bullet.knd = 6; // 弾の種類
                                    bullet.angle
                                        = bossatan3(bullet, GV.boss.x, GV.boss.y) + Math.PI - Math.PI / 6 + Math.PI / 6 * j; // 角度
                                    bullet.flag = 1;
                                    bullet.cnt = 0;
                                    bullet.state = 10 + j + 10 * child.state;
                                    bullet.spd = 0.4; // スピード

                                    MusicManager musicManager = MusicManager.GetInstance();
                                    musicManager.se_flag[0] = 1;
                                }
                            }
                            else if (t % 3 == 0) // それ以外の時で 3回に 1回
                            {
                                for (int j = 0; j < 3; j++) // 外向き3way
                                {
                                    int k = knum010;
                                    knum010++;

                                    Bullet bullet = GV.boss_shot.bullets[k]; // 注意! 弾を直接、指定している
                                    bullet.col = child.state; // 弾の色
                                    bullet.x = child.x; // 座標
                                    bullet.y = child.y;
                                    bullet.knd = 6; // 弾の種類
                                    bullet.angle
                                        = bossatan3(bullet, GV.boss.x, GV.boss.y) + Math.PI - Math.PI / 6 + Math.PI / 6 * j;//角度
                                    bullet.flag = 1;
                                    bullet.cnt = 0;
                                    bullet.state = 30;
                                    bullet.spd = 0.4; // スピード
                                }
                            }
                        }
                        // 消える時間なら消す
                        if (child.cnt > (int)CHILD_TIME1)
                            child.flag = 0;
                    }
                }
            }

            // 弾の処理
            for (int i = 0; i < GV.boss_shot.bullets.length(); i++)
            {
                Bullet bullet = GV.boss_shot.bullets[i];
                if (bullet.flag > 0)
                {
                    int cnt = bullet.cnt;
                    int state = bullet.state;

                    // 0は 1周目の内側発射弾、100は 2周目の内側発射弾
                    if (state == 0 || state == 100)
                    {
                        // 停止する時間なら止めてステータスを変える
                        if (bullet.cnt_till == cnt)
                        {
                            bullet.spd = 0.0;
                            bullet.state = 1;
                            if (state == 100)
                                bullet.state += 100;
                        }
                    }

                    // 1, 101は上記状態から止まった状態
                    if (state == 1 || state == 101)
                    {
                        // 動き出す時間なら動き出させてステータスを変える
                        if (bullet.cnt_stt == cnt)
                        {
                            bullet.state++;
                            if (flag010 == 0)
                                flag010 = 1;
                            bullet.cnt = 0;
                        }
                    }

                    // 2, 102は上記状態から加速し始めている(動いている)状態
                    if (state == 2 || state == 102)
                    {
                        if (flag010 == 1)
                            bullet.spd += 0.05;
                        if (bullet.spd > 2.0 && flag010 == 1)
                            flag010 = 2;

                        if (flag010 == 2)
                        {
                            if (state == 2)
                                if (bullet.spd < 2.0)
                                    bullet.spd += 0.05;
                            if (state == 102)
                                if (bullet.spd < 2.7)
                                    bullet.spd += 0.05;
                        }
                    }

                    // 11, 21はそれぞれ1周目、2周目の外側発射弾
                    if (state == 11 || state == 21)
                    {
                        if (cnt == RAG)
                        {
                            double zero_one = 0.0; // 初期値を設定する
                            double ang = 0.0;
                            double spd = 0.0;

                            if (num010 == 0)
                            {
                                zero_one = (double)(t - GOOUT010 - RAG - HANSHU) / CHILD_SHOT_TIME;
                                ang = ANG0 * zero_one;
                                spd = 2.3;
                            }
                            if (num010 == 1)
                            {
                                zero_one = (double)(tt1 - RAG - KAISHI010_1) / CHILD_SHOT_TIME1;
                                ang = Math.PI / 6 - (Math.PI / 6 + Math.PI / 4) * zero_one;
                                if (tt1 == 143)
                                    ang = Math.PI / 3;
                                spd = 2.3 * 0.55;
                            }

                            int k = knum010;
                            knum010++;

                            Bullet bullet2 = GV.boss_shot.bullets[k]; // 注意! 弾を直接、指定している
                            bullet2.col = (bullet.state == 11) ? 0 : 1; // 弾の色
                            bullet2.x = bullet.x; // 座標
                            bullet2.y = bullet.y;
                            bullet2.knd = 6; // 弾の種類
                            bullet2.angle = bossatan3(bullet2, GV.boss.x, GV.boss.y) + ang; // 角度
                            bullet2.flag = 1;
                            bullet2.cnt = 0;
                            if (num010 == 0)
                            {
                                bullet2.state = 0;
                                bullet2.cnt_till = (int)(150 * zero_one + 10);
                                bullet2.cnt_stt
                                    = (int)((ST0 - (ST_ED + TERM0)) * (1 - zero_one) + (ST_ED + TERM0) + 10);
                            }
                            if (num010 == 1)
                            {
                                bullet2.state = 100;
                                bullet2.cnt_till = (int)(150 * zero_one);
                                bullet2.cnt_stt
                                    = (int)((ST1 - (ST_ED1 + TERM1)) * (1 - zero_one) + (ST_ED1 + TERM1));
                            }
                            bullet2.spd = spd;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 011。フラッシュサンプル。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH011(BossShot boss_shot)
        {
            const int TM011 = 420;

            int t = boss_shot.cnt % TM011;
            int t2 = boss_shot.cnt;
            
            if (t2 == 60)
            {
                GV.flash.flag = 1;
                GV.flash.knd = 0; // 段々明るく、段々暗く
                GV.flash.to_cnt = 30; // 30カウントで変化
                GV.flash.cnt = 0;
            }
            if (t2 == 180)
            {
                GV.flash.flag = 1;
                GV.flash.knd = 1; // フラッシュ!
                GV.flash.to_cnt = 90; // 90カウントで変化
                GV.flash.cnt = 0;
            }
            if (t2 == 260)
            {
                GV.flash.flag = 1;
                GV.flash.knd = 0; // 段々明るく、段々暗く
                GV.flash.to_cnt = 15; // 15カウントで変化
                GV.flash.cnt = 0;
            }
            if (t2 == 380)
            {
                GV.flash.flag = 1;
                GV.flash.knd = 1; // フラッシュ!
                GV.flash.to_cnt = 30; // 30カウントで変化
                GV.flash.cnt = 0;
            }
        }

        /// <summary>
        /// 弾幕を処理します。必要に応じて弾を追加します。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        public static void do_boss_shot_bullet(BossShot boss_shot)
        {
            switch (GV.boss_infos[GV.boss.num].shot_knd[GV.boss.knd])
            {
                case 0:
                    boss_shot_bulletH000(boss_shot); // ノーマル
                    break;
                case 1:
                    boss_shot_bulletH002(boss_shot); // スペル～パーフェクトフリーズ～
                    break;
                case 2:
                    boss_shot_bulletH001(boss_shot); // ノーマル～サイレントセレナ～
                    break;
                case 3:
                    boss_shot_bulletH004(boss_shot); // スペル～ケロちゃん～
                    break;
                case 4:
                    boss_shot_bulletH009(boss_shot); // スペル～反魂蝶～
                    break;
                case 5:
                    boss_shot_bulletH000(boss_shot); // 原作版ではぬるぽ回避用
                    break;
                default:
                    Debug.WriteLine("boss.kndの値が不正(" + GV.boss.knd + ")です");
                    break;
            }
        }
    }
}
