﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// ボスの出現情報群を管理するクラスです。
    /// </summary>
    class BossInfoGroup
    {
        /// <summary>
        /// ボス情報群。
        /// </summary>
        private BossInfo[] boss_infos = new BossInfo[Setting.BOSS_NUM];

        /// <summary>
        /// 現在のステージに登場するボスの数。
        /// </summary>
        public int max_num;

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public BossInfoGroup()
        {
            for (int i = 0; i < this.boss_infos.Length; i++)
                this.boss_infos[i] = new BossInfo();
        }

        /// <summary>
        /// ボス情報にアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>ボス情報</returns>
        public BossInfo this[int index]
        {
            get
            {
                return this.boss_infos[index];
            }
        }

        /// <summary>
        /// ボス情報の数を返します。
        /// </summary>
        /// <returns>ボス情報の数</returns>
        public int length()
        {
            return this.boss_infos.Length;
        }

        /// <summary>
        /// 読み込みます。
        /// </summary>
        public void load()
        {
            string fname = Setting.BASE_DIR + "csv/bossH0.csv";
            int knd = 0; // 弾幕の種類
            int appear_count = -1; // ボスの出現カウンタ
            int lineNumber = 0; // ファイルの行数

            this.max_num = -1; // ボスの数

            DXFileReader reader = new DXFileReader();
            if(! reader.open(fname, 0))
            {
                Debug.WriteLine("ファイルオープンに失敗しました(" + fname + ")");
                return;
            }

            for (int i = 0; i < 2; i++) // 最初の 2行を読み飛ばす
            {
                reader.readLine();
                lineNumber++;
            }

            try {
                BossInfo bossInfo;

                while (true)
                {
                    string line = reader.readLine(); // 1行読み込む
                    lineNumber++;

                    if (line == null) // ファイルの末端に到達していれば
                        break; // ループから抜ける

                    if (line[0] == '/') // スラッシュがあれば
                        continue; // 読み飛ばす

                    string[] items = line.Split(new Char[] { ',' }); // 文字列を分解する

                    if (appear_count != int.Parse(items[0]))
                    {
                        if (this.max_num != -1)
                        {
                            bossInfo = this.boss_infos[this.max_num];
                            bossInfo.danmaku_num = knd;
                        }

                        appear_count = int.Parse(items[0]);
                        this.max_num++;
                        knd = 0;

                        // 情報を格納する
                        bossInfo = this.boss_infos[this.max_num];
                        bossInfo.appear_count = appear_count;
                    }
                    else
                        bossInfo = this.boss_infos[this.max_num];

                    // 情報を格納する
                    bossInfo.set_hp[knd] = int.Parse(items[1]);
                    bossInfo.shot_knd[knd] = int.Parse(items[2]);
                    bossInfo.back_knd[knd] = int.Parse(items[3]);

                    knd++;
                }

                // 読み終え時に最後のボスの弾幕数を設定する
                bossInfo = this.boss_infos[this.max_num];
                bossInfo.danmaku_num = knd;

                // ボスの数を設定する
                this.max_num++;
            }
            catch (FormatException)
            {
                Debug.WriteLine(lineNumber + "行目に不適切なデータが存在します");
            }

            // ファイルのクローズ
            if (!reader.close())
            {
                Debug.WriteLine("ファイルのクローズに失敗しました(" + fname + ")");
            }
        }
    }
}
