﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// 時間を測定するためのクラスです。
    /// </summary>
    class CheckTime
    {
        /// <summary>
        /// 調べられる関数の最大数。
        /// </summary>
        private const int FUNC_MAX = 30;

        /// <summary>
        /// 今、何個目かをカウントする。
        /// </summary>
        private static int func_count;

        /// <summary>
        /// 前回測定した時間。
        /// </summary>
        private static long lt;
        
        /// <summary>
        /// 測定データ群。
        /// </summary>
        private static FuncTm[] func_tm = new FuncTm[FUNC_MAX];

        /// <summary>
        /// 初期化します。
        /// </summary>
        public static void initialize()
        {
            for (int i = 0; i < func_tm.Length; i++)
                func_tm[i] = new FuncTm();
        }

        /// <summary>
        /// stと言う名前で受け取ったデータを登録します。flagが 1ならリセットします。
        /// </summary>
        /// <param name="st">名前</param>
        /// <param name="flag">フラグ。1のときリセット</param>
        public static void enter_func_tm(string st, int flag)
        {
            if (func_count >= FUNC_MAX)
            {
                Debug.WriteLine("func_countの値" + func_count + "が異常です\n");
                return;
            }
            long nowtm = DX.GetNowHiPerformanceCount() ;
            if (nowtm - lt < int.MaxValue) // intの扱える範囲なら
            {
                func_tm[func_count].tm = (int)(nowtm - lt); // 処理時間格納
                func_tm[func_count].str = st; // 文字列コピー
            }
            else // 扱えなかったら
                func_tm[func_count].tm = -1; // エラー

            lt = nowtm; // 前回記録した時刻として保存

            if(flag == 1) // リセット
            {
                for (int i = func_count + 1; i < func_tm.Length; i++)
                    func_tm[i].str = null;
                func_count = 0;
            }
            else
                func_count++;
        }

        /// <summary>
        /// stと言う名前で受け取ったデータを登録します。
        /// </summary>
        /// <param name="st">名前</param>
        public static void enter_func_tm(string st)
        {
            enter_func_tm(st, 0);
        }

        /// <summary>
        /// 浮動小数点数を「00.00」の形式で返します。
        /// </summary>
        /// <param name="value">浮動小数点数</param>
        /// <returns>00.00形式の文字列</returns>
        private static string convert_double(double value)
        {
            int i = (int)value;
            double d = value - i;

            return i.ToString("d2") + d.ToString("f2").Substring(1);
        }

        /// <summary>
        /// データを描画します。
        /// </summary>
        /// <param name="x">X座標</param>
        /// <param name="y">Y座標</param>
        public static void draw_func_tm(int x, int y)
        {
            int i;
            int total = 0;
            for(i = 0; i < func_tm.Length; i++)
            {
                if(func_tm[i].str == null)
                    break;
                DX.DrawStringToHandle(x, y + 14 * i, convert_double(func_tm[i].tm / 1000.0) + ":" + func_tm[i].str, GV.color[0], GV.font[0]);
                total += func_tm[i].tm;
            }
            DX.DrawStringToHandle(x, y + 14 * i, "合計:" + convert_double(total / 1000.0), GV.color[0], GV.font[0]);
        }
    }
}
