﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// オプションボンボンを管理するクラスです。
    /// </summary>
    class OptionBbGroup
    {
        /// <summary>
        /// オプションの相対 X座標。
        /// </summary>
        private const int OPTION_X = 25;
        
        /// <summary>
        /// オプションの相対 Y座標。
        /// </summary>
        private const int OPTION_Y = 35;

        /// <summary>
        /// アイテム群。
        /// </summary>
        private OptionBb[] option_bbs = new OptionBb[2];

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public OptionBbGroup()
        {
            for (int i = 0; i < this.option_bbs.Length; i++)
            {
                this.option_bbs[i] = new OptionBb();
            }
        }

        /// <summary>
        /// オプションボンボンにアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>オプションボンボン</returns>
        public OptionBb this[int index]
        {
            get
            {
                return this.option_bbs[index];
            }
        }

        /// <summary>
        /// オプションボンボンの数を返します。
        /// </summary>
        /// <returns>オプションボンボンの数</returns>
        public int length()
        {
            return this.option_bbs.Length;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.option_bbs.Length; i++)
                this.option_bbs[i].initialize();
        }

        /// <summary>
        /// ゲームの初期化をします。
        /// </summary>
        public void ini()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            this.option_bbs[0].img = imageManager.img_cshot[2];
            this.option_bbs[1].img = imageManager.img_cshot[2];
        }

        /// <summary>
        /// オプションボンボンを計算します。
        /// </summary>
        public void calc_option_bb()
        {
            if (GV.ch.power >= 100) // パワー 100以上ならオプションをつける
            {
                KeyManager keyManager = KeyManager.GetInstance();

                for (int i = 0; i < GV.option_bbs.length(); i++)
                {
                    if (keyManager.CheckStatePad(GV.configpad.slow) == 0) // 低速移動中じゃないなら
                    {
                        this.option_bbs[i].x = GV.ch.x + OPTION_X * (i == 0 ? -1 : 1);
                        this.option_bbs[i].y = GV.ch.y + OPTION_Y + Math.Sin(MyMath.PI2 / 150 * this.option_bbs[i].cnt) * 20;
                    }
                    else
                    {
                        this.option_bbs[i].x = GV.ch.x + OPTION_X / 2 * (i == 0 ? -1 : 1);
                        this.option_bbs[i].y = GV.ch.y + OPTION_Y / 2;
                    }
                    this.option_bbs[i].cnt++;
                }

                // オプションを有効にする
                this.option_bbs[0].flag = 1;
                this.option_bbs[1].flag = 1;
            }
            else
            {
                // オプションを無効にする
                this.option_bbs[0].flag = 0;
                this.option_bbs[1].flag = 0;
            }
        }
    }
}
