﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 弾を表すクラスです。
    /// </summary>
    class Bullet
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 色。
        /// </summary>
        public int col;

        /// <summary>
        /// 状態。
        /// </summary>
        public int state;

        /// <summary>
        /// 少なくとも消さない時間。
        /// </summary>
        public int till;

        /// <summary>
        /// エフェクトの種類。
        /// </summary>
        public int eff;

        /// <summary>
        /// 回転するかどうかのフラグ。
        /// 0=回転しない、1=回転する。
        /// </summary>
        public int kaiten;

        /// <summary>
        /// エフェクトの詳細。
        /// </summary>
        public int eff_detail;

        /// <summary>
        /// 弾が一時停止する時間(弾幕 H010用)。
        /// </summary>
        public int cnt_till;

        /// <summary>
        /// 弾が動き出す時間(弾幕 H010用)。
        /// </summary>
        public int cnt_stt;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;

        /// <summary>
        /// Y座標。
        /// </summary>
        public double vy;

        /// <summary>
        /// X速度。
        /// </summary>
        public double vx;

        /// <summary>
        /// Y速度。
        /// </summary>
        public double y;

        /// <summary>
        /// 角度。
        /// </summary>
        public double angle;

        /// <summary>
        /// スピード。
        /// </summary>
        public double spd;

        /// <summary>
        /// ベースの角度。
        /// </summary>
        public double[] base_angle = new double[1];

        /// <summary>
        /// 一次記憶スピード。
        /// </summary>
        public double[] rem_spd = new double[1];

        /// <summary>
        /// ?
        /// </summary>
        public int c_flag;

        /// <summary>
        /// ?
        /// </summary>
        public double cx;

        /// <summary>
        /// ?
        /// </summary>
        public double cy;

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.knd = 0;
            this.cnt = 0;
            this.col = 0;
            this.state = 0;
            this.till = 0;
            this.eff = 0;
            this.kaiten = 0;
            this.eff_detail = 0;
            this.cnt_till = 0;
            this.cnt_stt = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.vx = 0.0;
            this.vy = 0.0;
            this.angle = 0.0;
            this.spd = 0.0;
            for (int i = 0; i < this.base_angle.Length; i++)
                this.base_angle[i] = 0.0;
            for (int i = 0; i < this.rem_spd.Length; i++)
                this.rem_spd[i] = 0.0;
            this.c_flag = 0;
            this.cx = 0.0;
            this.cy = 0.0;
        }

        /// <summary>
        /// 計算します。
        /// </summary>
        public void calc()
        {
            this.x += Math.Cos(this.angle) * this.spd;
            this.y += Math.Sin(this.angle) * this.spd;
            this.cnt++;

            if (this.x < -50 || Setting.FMX + 50 < this.x || this.y < -50 || Setting.FMY + 50 < this.y) // 画面から外れたら
                if (this.till < this.cnt) // 最低消えない時間より長ければ
                    this.flag = 0; // 消す
        }

        /// <summary>
        /// 自キャラクタに当たったかどうか判定する。
        /// </summary>
        /// <param name="ch">自キャタクタ</param>
        /// <returns>当たった場合は true</returns>
        private bool out_judge_enemyshot(Ch ch)
        {
            if (this.cnt > 0) // ショットの軌道が一度でも計算されていたら
            {
                return MyMath.out_judge(this.x, this.y, ch.x, ch.y, GV.bullet_infos[this.knd].range, Setting.CRANGE, this.spd, this.angle, GV.bullet_infos[this.knd].length);
            }
            return false;
        }
        
        /// <summary>
        /// 弾が自キャラクタに当たる処理をします。
        /// </summary>
        /// <returns>自キャラクタが死んだとき true</returns>
        public bool enemyshot_and_ch()
        {
            if (GV.bom.flag != 0) // ボムが発射されていたら
            {
                this.flag = 0; // 弾をオフ
                return false;
            }
            if (this.out_judge_enemyshot(GV.ch)) // 自キャラクタに弾が接触したら
            {
                this.flag = 0; // 弾をオフ
                if (GV.ch.flag == 0 && GV.ch.mutekicnt == 0) // ステータスが通常で、無敵でなかったら
                {
                    GV.ch.setCryBomState(); // 自キャラクタを喰らいボム状態にする
                    return true;
                }
            }

            return false;
        }
    }
}
